/**
 * Champion Medals
 * Copyright (c) 2023 NaNInf
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

class MedalWindow {
    MedalWindow() {}

    void Draw() {
        if (!setting_window_enable || playground_map is null || !playground_map.has_cm ||
            (setting_window_hide_with_interface && !UI::IsGameUIVisible())) return;

        int window_flags = UI::WindowFlags::NoTitleBar |
                           UI::WindowFlags::NoResize |
                           UI::WindowFlags::NoScrollbar |
                           UI::WindowFlags::NoScrollWithMouse |
                           UI::WindowFlags::AlwaysAutoResize |
                           UI::WindowFlags::NoDocking;

        if (!UI::IsOverlayShown()) window_flags |= UI::WindowFlags::NoMove;

        UI::Begin("Champion Medals", window_flags);

        // only show PB/delta when those settings are checked and a PB is available
        bool pb_available = playground_map.pb_time != uint(-1) && playground_map.pb_time != uint(-1) - 1;
        bool show_pb = setting_window_show_pb && pb_available;
        bool show_pb_delta = setting_window_show_delta && pb_available;

        int table_columns = 3 + (show_pb_delta ? 1 : 0);

        if (UI::BeginTable("##main", table_columns)) {
            if (show_pb && playground_map.cm_awarded()) {
                TableRowPB(show_pb_delta);
            }
            TableRowChampion(show_pb_delta && !show_pb);
            if (show_pb && !playground_map.cm_awarded()) {
                TableRowPB(show_pb_delta);
            }

            UI::EndTable();
        }

        UI::End();
    }

    void TableRowChampion(bool show_pb_delta) {
        UI::TableNextRow();

        UI::TableNextColumn();
        medal.WindowImage(UI::GetScale() * vec2(16, 16));

        UI::TableNextColumn();
        UI::Text("Champion");

        UI::TableNextColumn();
        UI::Text(Time::Format(playground_map.cm_time));

        if (show_pb_delta) TableColumnDelta();
    }

    void TableRowPB(bool show_pb_delta) {
        UI::TableNextRow();

        UI::TableNextColumn();
        UI::TableNextColumn();
        UI::Text("Pers. Best");

        UI::TableNextColumn();
        UI::Text(Time::Format(playground_map.pb_time));

        if (show_pb_delta) TableColumnDelta();
    }

    void TableColumnDelta() {
        UI::TableNextColumn();
        int delta = int(playground_map.pb_time) - int(playground_map.cm_time);
        if (delta <= 0) {
            UI::Text("\\$77f\u2212" + Time::Format(-delta));
        } else {
            UI::Text("\\$f77+" + Time::Format(delta));
        }
    }
}
