/**
 * Champion Medals
 * Copyright (c) 2023 NaNInf
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

class CampaignLayer : Layer {
    string identifier;
    string previous_identifier;

    protected Campaign@ campaign;

    bool IsVisible() override {
        CGameCtnApp@ app = GetApp();
        return ui_layer !is null && ui_layer.IsVisible && app.Network.ClientManiaAppPlayground is null;
    }

    void CheckUpdate() override {
        if (!IsVisible()) {
            @campaign = null;
            previous_identifier = identifier = "";
            return;
        }
        identifier = GetIdentifier();
        if (identifier != previous_identifier) Update();
        previous_identifier = identifier;
    }

    private void Update() override {
        log("Updating campaign layer, " + identifier, LogLevel::Print);
        @campaign = null;
        @campaign = Api::GetCampaign(identifier);
    }

    private bool ShouldDraw() override {
        if (LoadingVisible() || CloseDialogVisible() || !IsVisible()) return false;
        if (campaign is null || !campaign.has_cms) return false;
        // check if any track has visible medals
        if (ui_layer is null || ui_layer.LocalPage.GetFirstChild("frame-maps") is null) return false;
        CGameManialinkFrame@ maps = cast<CGameManialinkFrame@>(ui_layer.LocalPage.GetFirstChild("frame-maps"));
        MwFastBuffer<CGameManialinkControl@> controls = maps.Controls;
        for (uint i = 0; i < controls.Length; ++i) {
            if (controls[i].Visible &&
                cast<CGameManialinkFrame@>(controls[i]).GetFirstChild("frame-medalstack") !is null &&
                cast<CGameManialinkFrame@>(controls[i]).GetFirstChild("frame-medalstack").Visible) return true;
        }
        return false;
    }
}
