/**
 * Champion Medals
 * Copyright (c) 2023 NaNInf
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

class Layer {
    CGameUILayer@ ui_layer = null;

    Layer() {}

    void SetUILayer(CGameUILayer@ _ui_layer) final {
        @ui_layer = _ui_layer;
    }

    bool UILayerIsNull() final {
        return ui_layer is null;
    }

    bool IsVisible() {
        return !LoadingVisible() && ui_layer !is null && ui_layer.IsVisible;
    }

    void CheckUpdate() {}

    void Draw() {
        if (!ShouldDraw()) return;
    }

    string GetIdentifier() {
        return "";
    }

    bool ShouldDraw() {
        return true;
    }

    protected CGameManialinkFrame@ GetFrame(const string &in control_id) final {
        return (ui_layer !is null) ? cast<CGameManialinkFrame@>(ui_layer.LocalPage.GetFirstChild(control_id)) : null;
    }

    protected string GetLabelValue(const string &in control_id) final {
        if (ui_layer is null) return "";
        CGameManialinkLabel@ label = cast<CGameManialinkLabel@>(ui_layer.LocalPage.GetFirstChild(control_id));
        if (label is null) return "";
        return label.Value;
    }

    protected string GetLabelValue(CGameManialinkFrame@ frame, const string &in control_id) final {
        if (frame is null) return "";
        CGameManialinkLabel@ label = cast<CGameManialinkLabel@>(frame.GetFirstChild(control_id));
        if (label is null) return "";
        return label.Value;
    }

    protected void Update() {}

    protected bool LoadingVisible() final {
        auto load_progress = GetApp().LoadProgress;
        return load_progress !is null && load_progress.State != NGameLoadProgress::EState::Disabled;
    }

    protected bool CloseDialogVisible() final {
        return layers.close_dialog_layer !is null && layers.close_dialog_layer.IsVisible();
    }
}
