/**
 * Champion Medals
 * Copyright (c) 2023 NaNInf
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

namespace LayerManager {
    void UpdateLayers() {
        UpdateMenuLayers();
        UpdatePlaygroundLayers();
        UpdateCloseDialogLayer();
    }

    void UpdateMenuLayers() {
        CTrackMania@ app = cast<CTrackMania@>(GetApp());

        // menu layers don't have to be checked if a playground exists
        if (app.Network.ClientManiaAppPlayground !is null ||
            app.MenuManager.MenuCustom_CurrentManiaApp is null) {
            layers.regular_campaign_layer.SetUILayer(null);
            layers.regular_campaign_live_layer.SetUILayer(null);
            layers.monthly_campaign_layer.SetUILayer(null);
        } else {
            auto ui_layers = app.MenuManager.MenuCustom_CurrentManiaApp.UILayers;

            array<CGameUILayer@> regular_campaign_ui_layers;
            array<CGameUILayer@> regular_campaign_live_ui_layers;
            array<CGameUILayer@> regular_official_campaign_live_ui_layers;
            array<CGameUILayer@> monthly_campaign_ui_layers;

            for (uint i = 0; i < ui_layers.Length; ++i) {
                CGameUILayer@ ui_layer = ui_layers[i];

                string header = ui_layer.ManialinkPageUtf8.SubStr(0, 100);

                if (header.IndexOf("\"Page_CampaignDisplay\"") != -1) {
                    regular_campaign_ui_layers.InsertLast(ui_layer);
                }
                if (header.IndexOf("\"Page_RoomMapListDisplay\"") != -1) {
                    regular_campaign_live_ui_layers.InsertLast(ui_layer);
                }
                if (header.IndexOf("\"Page_RoomCampaignDisplay\"") != -1) {
                    regular_official_campaign_live_ui_layers.InsertLast(ui_layer);  // TODO CHECK ALIGNMENT OF THIS SHIT
                }
                if (header.IndexOf("\"Page_MonthlyCampaignDisplay\"") != -1) {
                    monthly_campaign_ui_layers.InsertLast(ui_layer);
                }
            }

            if (regular_campaign_ui_layers.Length >= 1) {
                if (regular_campaign_ui_layers.Length > 1)
                    log("More than one regular campaign layer found!", LogLevel::Error);
                layers.regular_campaign_layer.SetUILayer(regular_campaign_ui_layers[0]);
            }
            if (regular_campaign_live_ui_layers.Length >= 1) {
                if (regular_campaign_live_ui_layers.Length > 1)
                    log("More than one regular campaign live layer found!", LogLevel::Error);
                layers.regular_campaign_live_layer.SetUILayer(regular_campaign_live_ui_layers[0]);
            }
            if (regular_official_campaign_live_ui_layers.Length >= 1) {
                if (regular_official_campaign_live_ui_layers.Length > 1)
                    log("More than one regular official campaign live layer found!", LogLevel::Error);
                layers.regular_official_campaign_live_layer.SetUILayer(regular_official_campaign_live_ui_layers[0]);
            }
            if (monthly_campaign_ui_layers.Length >= 1) {
                if (monthly_campaign_ui_layers.Length > 1)
                    log("More than one monthly campaign layer found!", LogLevel::Error);
                layers.monthly_campaign_layer.SetUILayer(monthly_campaign_ui_layers[0]);
            }
        }
    }

    void UpdatePlaygroundLayers() {
        CTrackMania@ app = cast<CTrackMania@>(GetApp());

        if (app.Network.ClientManiaAppPlayground is null) {
            layers.start_menu_layer.SetUILayer(null);
            layers.pause_menu_layer.SetUILayer(null);
            layers.end_menu_layer.SetUILayer(null);
            layers.local_end_menu_layer.SetUILayer(null);
            layers.live_pause_menu_layer.SetUILayer(null);
            layers.live_record_layer.SetUILayer(null);
            layers.live_new_pb_layer.SetUILayer(null);
            layers.scores_table_layer.SetUILayer(null);
        } else {
            // solo and live modes have different playground layers
            bool solo = cast<CGameCtnNetServerInfo>(app.Network.ServerInfo).ServerLogin == "";

            auto ui_layers = app.Network.ClientManiaAppPlayground.UILayers;

            array<CGameUILayer@> start_menu_ui_layers;
            array<CGameUILayer@> pause_menu_ui_layers;
            array<CGameUILayer@> end_menu_ui_layers;
            array<CGameUILayer@> local_end_menu_ui_layers;
            array<CGameUILayer@> live_pause_menu_ui_layers;
            array<CGameUILayer@> live_record_ui_layers;

            array<CGameUILayer@> scores_table_ui_layers;

            for (uint i = 0; i < ui_layers.Length; ++i) {
                CGameUILayer@ ui_layer = ui_layers[i];

                string header = ui_layer.ManialinkPageUtf8.SubStr(0, 100);

                if (header.IndexOf("\"UIModule_Campaign_StartRaceMenu\"") != -1) {
                    start_menu_ui_layers.InsertLast(ui_layer);
                }
                if (header.IndexOf("\"UIModule_Campaign_PauseMenu\"") != -1 ||
                    header.IndexOf("\"UIModule_PlayMap_PauseMenu\"") != -1) {
                    pause_menu_ui_layers.InsertLast(ui_layer);
                }
                if (header.IndexOf("\"UIModule_Campaign_EndRaceMenu\"") != -1) {
                    end_menu_ui_layers.InsertLast(ui_layer);
                }
                if (header.IndexOf("\"UIModule_PlayMap_EndRaceMenu\"") != -1) {
                    local_end_menu_ui_layers.InsertLast(ui_layer);
                }
                if (header.IndexOf("\"UIModule_Online_PauseMenu\"") != -1) {
                    live_pause_menu_ui_layers.InsertLast(ui_layer);
                }
                if (header.IndexOf("\"UIModule_Race_Record\"") != -1) {
                    live_record_ui_layers.InsertLast(ui_layer);
                }

                if (header.IndexOf("\"UIModule_Race_ScoresTable\"") != -1) {
                    scores_table_ui_layers.InsertLast(ui_layer);
                }
            }

            if (solo) {
                if (start_menu_ui_layers.Length >= 1) {
                    if (start_menu_ui_layers.Length > 1)
                        log("More than one start menu layer found!", LogLevel::Error);
                    layers.start_menu_layer.SetUILayer(start_menu_ui_layers[0]);
                }
                if (pause_menu_ui_layers.Length >= 1) {
                    if (pause_menu_ui_layers.Length > 1)
                        log("More than one pause menu layer found!", LogLevel::Error);
                    layers.pause_menu_layer.SetUILayer(pause_menu_ui_layers[0]);
                }
                if (end_menu_ui_layers.Length >= 1) {
                    if (end_menu_ui_layers.Length > 1)
                        log("More than one end menu layer found!", LogLevel::Error);
                    layers.end_menu_layer.SetUILayer(end_menu_ui_layers[0]);
                }
                if (local_end_menu_ui_layers.Length >= 1) {
                    if (local_end_menu_ui_layers.Length > 1)
                        log("More than one local end menu layer found!", LogLevel::Error);
                    layers.local_end_menu_layer.SetUILayer(local_end_menu_ui_layers[0]);
                }
            } else {
                if (live_pause_menu_ui_layers.Length >= 1) {
                    if (live_pause_menu_ui_layers.Length > 1)
                        log("More than one live pause menu layer found!", LogLevel::Error);
                    layers.live_pause_menu_layer.SetUILayer(live_pause_menu_ui_layers[0]);
                }
                if (live_record_ui_layers.Length >= 1) {
                    if (live_record_ui_layers.Length > 1)
                        log("More than one live record layer found!", LogLevel::Error);
                    layers.live_record_layer.SetUILayer(live_record_ui_layers[0]);
                    layers.live_new_pb_layer.SetUILayer(live_record_ui_layers[0]);
                }
            }

            if (scores_table_ui_layers.Length >= 1) {
                if (scores_table_ui_layers.Length > 1)
                    log("More than one scores table layer found!", LogLevel::Error);
                layers.scores_table_layer.SetUILayer(scores_table_ui_layers[0]);
            }
        }
    }

    void UpdateCloseDialogLayer() {
        CHmsViewport@ viewport = GetApp().Viewport;

        for (uint i = 0; i < viewport.Overlays.Length; ++i) {
            CHmsZoneOverlay@ overlay = viewport.Overlays[i];
            if (overlay.m_CorpusVisibles.Length == 0) continue;

            CControlContainer@ scene_mobil = cast<CControlContainer@>(overlay.m_CorpusVisibles[0].Item.SceneMobil);
            if (scene_mobil is null) continue;
            if (scene_mobil.IdName != "FrameConfirmQuit") continue;

            layers.close_dialog_layer.SetOverlay(viewport.Overlays[i]);
            return;
        }

        layers.close_dialog_layer.SetOverlay(null);
    }
}
